/* Copyright 1994, 1995 David Koski, Cjin Pheow Lee, Barton Miller
 * All rights reserved
 *
 * This software is furnished under the condition that it may not
 * be provided or otherwise made available to, or used by, any
 * other person.  No title to or ownership of the software is
 * hereby transferred.
 *
 * Any use of this software must include the above copyright notice.
 *
 * Portjig Library - Fuzz Testing Tools
 *
 * File: udp.c 
 * Last Update: April 4, 1995 
 *
 * Description: This program is the UDP part of the Portjig test library.
 *              It tests udp-connection network services. Input is to the
 *              program is fed from stdin.
 *
 * Usage: udp <hostname> <port number>
 */

static char Copyright[] =
"@(#) Copyright 1994, 1995 David Koski, Cjin Lee, Barton Miller.\n\
 All rights reserved.\n";

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <ctype.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <time.h>

#define SENDSIZ 16 /* data send to and receive from servers in 16-byte chucks */

/* Initialize local socket */
int client_init(void);

/* Initialize remote socket */
int remote_init(struct sockaddr_in *remote, char *hostname, int port);

main(int argc, char *argv[])
{
  int csock;
  int count=0;
  struct sockaddr_in remote;
  char sendbuf[BUFSIZ], recvbuf[BUFSIZ];
  int rlen = sizeof remote;

  memset(sendbuf,0,BUFSIZ);
  memset(recvbuf,0,BUFSIZ);

  if (argc!=3) {
    fprintf(stdout, "Portjig Library - Fuzz Testing Tools\n");
    fprintf(stdout, "Usage: %s <hostname> <port number>\n", argv[0]);
    exit(0);
  }

  csock = client_init();

  if (!remote_init(&remote, argv[1], atoi(argv[2]))) {
    fprintf(stdout, "remote service not available.\n");
    exit(0);
  }

  /* make socket non-blocking - so that a recv on it will return right away */
  if (fcntl(csock,F_SETFL,FNDELAY)<0) {
    perror("fcntl");
    exit(1);
  }

  while (1) {
    int len=0, i;

    memset(recvbuf,0,BUFSIZ);

    /* non-block recv from remote */
    len=recvfrom(csock, recvbuf, BUFSIZ, 0, &remote, &rlen);
    if (len<0 && errno!=EWOULDBLOCK) {
      perror("recvfrom");
      exit(1);
    }
    for (i=0;i<len;i++)
	  fprintf(stdout, "%c", recvbuf[i]);

    fflush(stdout);

    memset(sendbuf,0,SENDSIZ);

    len = read(fileno(stdin),sendbuf,BUFSIZ);
    if (len>0) {
      if ((len=sendto(csock, sendbuf, len, 0, &remote, sizeof remote))==-1) {
	if (errno!=EWOULDBLOCK) {
	  perror("sendto");
	  exit(1);
	}
      }
    }
    else {
      fprintf(stdout, "Done.\n");
      break;
    }
  }
  close(csock);
}


int client_init(void)
{
  struct sockaddr_in client;
  int sock;

  memset((void *) &client,0, sizeof client);
  client.sin_family = AF_INET;
  client.sin_addr.s_addr = INADDR_ANY;
  client.sin_port = htons(0);

  if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    perror("socket");
    exit(1);
  }

  if (bind(sock, (struct sockaddr *)&client, sizeof client) < 0) {
    perror("bind");
    exit(1);
  }
  return sock;
}

int remote_init(struct sockaddr_in *remote, char *hostname, int port)
{
  int sock;
  struct hostent *hostp;

  memset((char *)remote, 0, sizeof(struct sockaddr));
  
  if ((hostp = gethostbyname(hostname)) == 0) {
    fprintf(stderr, "%s: unknown host\n", hostname);
    return 0;
  }

  remote->sin_family = AF_INET;
  memcpy((char *)&remote->sin_addr, hostp->h_addr, hostp->h_length);
  remote->sin_port = htons((u_short)port);
  return 1;
}










